/*
 * This file is part of the LibreOffice project.
 *
 * Based on LLVM/Clang.
 *
 * This file is distributed under the University of Illinois Open Source
 * License. See LICENSE.TXT for details.
 *
 */

#include "plugin.hxx"

#include <clang/AST/ASTConsumer.h>
#include <clang/Frontend/CompilerInstance.h>
#include <clang/Frontend/FrontendAction.h>
#include <clang/Frontend/FrontendPluginRegistry.h>
#include <clang/Rewrite/Rewriter.h>

#include "createinstancecheck.hxx"
#include "postfixincrementfix.hxx"
#include "unusedstringcheck.hxx"

using namespace clang;

namespace loplugin
{

class AstConsumer
    : public ASTConsumer
    {
    public:
        explicit AstConsumer( ASTContext& context )
            : rewriter( context.getSourceManager(), context.getLangOpts())
            , createInstanceCheck( context )
            , postfixIncrementFix( context, rewriter )
            , unusedStringCheck( context )
            {
            }
        virtual void HandleTranslationUnit( ASTContext& context )
            {
            if( context.getDiagnostics().hasErrorOccurred())
                return;
#if 1
            createInstanceCheck.run();
#endif
#if 1
            postfixIncrementFix.run();
#endif
#if 1
            unusedStringCheck.run();
#endif
            // TODO also LO header files? or a subdir?
           if( const RewriteBuffer* buf = rewriter.getRewriteBufferFor( context.getSourceManager().getMainFileID()))
                buf->write( llvm::outs());
            // TODO else write out the original file?
            }
    private:
        Rewriter rewriter;
        CreateInstanceCheck createInstanceCheck;
        PostfixIncrementFix postfixIncrementFix;
        UnusedStringCheck unusedStringCheck;
    };

class LibreOfficeAction
    : public PluginASTAction
    {
    public:
        virtual ASTConsumer* CreateASTConsumer( CompilerInstance& Compiler, StringRef InFile )
            {
            return new AstConsumer( Compiler.getASTContext());
            }
        virtual bool ParseArgs( const CompilerInstance& CI, const std::vector< std::string >& args )
            {
            return true;
            }
    };

} // namespace

static FrontendPluginRegistry::Add< loplugin::LibreOfficeAction > X( "lotest", "LO test" );
