/*
 * This file is part of the LibreOffice project.
 *
 * Based on LLVM/Clang.
 *
 * This file is distributed under the University of Illinois Open Source
 * License. See LICENSE.TXT for details.
 *
 */

#include "unusedstringcheck.hxx"

#include <clang/Basic/SourceManager.h>

using namespace clang;
using namespace std;

#include <iostream>

namespace loplugin
{

UnusedStringCheck::UnusedStringCheck( ASTContext& context )
    : context( context )
    {
    }

void UnusedStringCheck::run()
    {
    TraverseDecl( context.getTranslationUnitDecl());
    }

bool UnusedStringCheck::VisitNamedDecl( NamedDecl* declaration )
    {
    // TODO also LO header files? or a subdir?
    if( !context.getSourceManager().isFromMainFile( declaration->getLocStart()))
        return true;
    if( !isa< VarDecl >( declaration ))
        return true;
    VarDecl* var = cast< VarDecl >( declaration );
    if( CXXRecordDecl* type = var->getType()->getAsCXXRecordDecl())
        if( type->getQualifiedNameAsString() == "rtl::OUString" )
            cout << "S:" << var->getDeclName().getAsString()
                << ":" << var->isReferenced() << ":" << var->isUsed() << endl;
    return true;
    }

} // namespace
